{% extends "base.html" %}

{% block head %}
  <style type="text/css">
    ul.archive {
      list-style-type: none;
      margin: 0;
      padding: 0;
    }

    ul.archive li {
      margin-bottom: 1em;
    }

    ul.archive .title {
      font-family: "Helvetica Nue", Helvetica, Arial, sans-serif;
      font-size: 14pt;
    }
  </style>
{% end %}

{% block body %}
  <ul class="archive">
    {% for entry in entries %}
      <li>
        <div class="title"><a href="/entry/{{ entry.slug }}">{{ entry.title }}</a></div>
        <div class="date">{{ locale.format_date(entry.published, full_format=True, shorter=True) }}</div>
      </li>
    {% end %}
  </ul>
{% end %}
