{% extends "base.html" %}

{% block body %}
{% if error %}
<span style="color: red">Error: {{ error }}</span><p>
{% end %}

<form action="/auth/login" method="POST">
  Email: <input name="email" type="text"><br>
  Password: <input name="password" type="password"><br>
  {% module xsrf_form_html() %}
  <input type="submit">
</form>
{% end %}
